#!/bin/bash

conf_file=/etc/samba/policy.conf/software_install_user.conf
local_pkg_path=/var/lib/samba/software_install
local_deb_path=$local_pkg_path/deb
local_win_path=$local_pkg_path/win
local_ver_path=$local_pkg_path/version
local_log_path=$local_pkg_path/log
local_file_ver=$local_ver_path/software_install_user.version
local_file_ver_bk=$local_ver_path/software_install_user.version.bk
local_log=$local_log_path/software_install_user.log
flag_txt=txt

#get username
for param in $@
do
    tmp=${param%%=*}
    if [ "$tmp" = "--username" ];then
        username=${param#*=}
    fi
done

#get domainname
smbconf=`smbd -b | grep CONFIGFILE | cut -d : -f 2`
domainname=`grep -rnw 'realm' $smbconf | sed 's/ //g' | cut -d = -f 2`


if [ ! -d $local_ver_path ];then 
    mkdir -p $local_ver_path
fi

if [ ! -d $local_log_path ];then
    mkdir -p $local_log_path
fi


#create log
cat /dev/null > $local_log

exec 1>$local_log
exec 2>$local_log

if [ ! -f $conf_file ];then
    echo "$conf_file not found, exit"
    exit
fi

#if there no version number gived,exit
for param in `cat $conf_file`
do
    tmp=${param%%=*}
    if [ "$tmp" = "--VerNum" ];then
	net_ver=${param#*=}
    elif [ "$tmp" = "--SharedPath" ];then
	sharedpath=${param#*=}
    elif [ "$tmp" = "--RepoApps" ];then
        repoapps=${param#*=}
    elif [ "$tmp" = "--DebApps" ];then
        debapps=${param#*=}
    elif [ "$tmp" = "--WinApps" ];then
        winapps=${param#*=}
    fi
done

#verify whether net_ver meets the requirements or not
if [ -z "$net_ver" ] || [ -z "$(echo $net_ver | sed -n "/^[0-9]\+$/p")" ];then
    echo "vernum does not meet the requirements,it sould be a number, exit"
    if [ -f $local_file_ver ];then
        rm -rf $local_file_ver
    fi
    exit
fi

#verify whether any package need to be installed or not
if [ -z "$repoapps" ] && [ -z "$debapps" ] && [ -z "$winapps" ];then
    echo "no package need to be installed, exit."
    exit
fi


#determine if there is a policy update based on the version number
if [ -f $local_file_ver ];then
    local_ver=`cat $local_file_ver`
    echo "local_ver = $local_ver, net_ver = $net_ver"
    if [[ $net_ver = $local_ver ]];then
        echo "***stage1 : there is nothing changed, exit!**"
        exit
    elif [[ $net_ver != $local_ver ]];then
        echo "*****stage1 : something have changed!*******"
        echo $net_ver > $local_file_ver

	#for handling exception
	mv $local_file_ver $local_file_ver_bk
    fi
elif [ -f $local_file_ver_bk ];then
    echo "***after power off exception, create version file***"
    echo $net_ver > $local_file_ver
    after_exception=1

    #for handling power off exception
    mv $local_file_ver $local_file_ver_bk
    
else
    echo "****stage1: there no version file, create it!***"
    echo $net_ver > $local_file_ver

    #for handling exception
    mv $local_file_ver $local_file_ver_bk
fi


##handle repoapps
if [ -n $repoapps ];then
    repoapps=${repoapps//:/ }
    for repo_pkg in $repoapps
    do
        apt-get -y install $repo_pkg
	repo_installed_pkgs=$repo_installed_pkgs':'$repo_pkg
    done
fi


net_deb_path=$sharedpath\\deb
net_win_path=$sharedpath\\win

if [[ -n $debapps ]];then
    debpkgs=${debapps//:/ }
    for pkg in $debpkgs
    do
        file_path=$local_deb_path/$pkg
	if [ ! -d $file_path ];then
	    mkdir -p $file_path
	fi
    done

    if [ -n "$sharedpath" ];then

        #pull all lists.txt files needed from remote server
	result=$(net ads gpo pull -P $username $domainname $net_deb_path $debapps $local_pkg_path $flag_txt)
	
	res=`echo $result | grep -w "gpo_fetch_packages failed"`
	if [ -n "$res" ];then
            echo "pull lists files failure, please check that sharedpath is correct or not!"
            mv $local_file_ver_bk $local_file_ver
	    exit
	fi
        echo "*****stage2: pull all lists.txt files sucessfully*****"
    else
	echo "sharedpath is null, we can't pull txt files, exit!"
	mv $local_file_ver_bk $local_file_ver
	exit
    fi

    for pkg in $debpkgs
    do
        txt_file=$local_deb_path/$pkg/lists.txt

	if [ -f "$txt_file" ];then
	    deb=`cat $txt_file`

	    need_install=0
	    need_pull=0
	    for line in `cat $txt_file`
            do
		debname=${line%%_*}
		installed_name=`dpkg -l | awk '{print $2}' | grep -w "$debname"`
		if [ -z "$installed_name" ];then
		    echo "uninstalled package"
		    need_install=1
		    local_pkg=`find $local_deb_path/$pkg -type f -name "$line"`
		    if [ -z "$local_pkg" ];then
			need_pull=1
		    elif [[ $after_exception -eq 1 ]];then
		        need_pull=1
		    fi
		else
		    vernum=`dpkg -l | awk '{print $2, $3}' | grep -w "$debname" | grep -v "$debname-" | awk '{print $2}'`
		    txt_vernum=`dpkg-deb -f $local_deb_path/$pkg/$line | grep -w "Version:" | awk '{print $2}'`
                    echo "txt_vernum = $txt_vernum, vernum = $vernum"
		    if [ "$txt_vernum" != "$vernum" ];then
		        need_install=1
			rep_pkg=$debname'_'$txt_vernum'('$vernum')'
			replaced_pkgs=$replaced_pkgs';'$rep_pkg

			diff_pkgs=$diff_pkgs':'$debname
			local_pkg=`find $local_deb_path/$pkg -type f -name "$line"`
			if [ -z "$local_pkg" ] || [[ $after_exception -eq 1 ]];then
			    need_pull=1
			fi
		    elif [[ $after_exception -eq 1 ]];then
		        need_install=1
			need_pull=1
		    else
			echo "$installed_name has been installed."
		    fi
		fi
	    done

		
	    if [ $need_install -eq 1 ];then
		uninstalled_pkgs=$uninstalled_pkgs":"$pkg
	    elif [ $need_install -eq 0 ];then
	        continue;
	    fi

	    if [ $need_pull -eq 1 ];then
	  	pull_pkgs=$pull_pkgs":"$pkg
	    fi

	    if [ -z "$sharedpath" ];then
                if [ -z "$pull_pkgs" ] && [ -n "$uninstalled_pkgs" ];then
		    if [ -n "$diff_pkgs" ];then
			diff_pkgs=${diff_pkgs#*:}
			new_diff=${diff_pkgs//:/ }
			for debpkg in $new_diff
			do
			    apt-get -y purge $debpkg
			    echo "remove package: $debpkg"
			done
		    fi
		fi
	    fi
	else
	    echo "lists.txt file not found, exit!"
	    mv $local_file_ver_bk $local_file_ver
	    exit
	fi
    done

    repo_installed_pkgs=${repo_installed_pkgs#*:}
    uninstalled_pkgs=${uninstalled_pkgs#*:}
    replaced_pkgs=${replaced_pkgs#*;}
    pull_pkgs=${pull_pkgs#*:}

    #if all packages have been installed, exit directly!
    if [ -z "$uninstalled_pkgs" ];then
        mv $local_file_ver_bk $local_file_ver
	echo "all packages have been installed, exit."
	exit 0
    fi
 
    #pull deb packages from remote server
    if [ -n "$sharedpath" ] && [ -n "$pull_pkgs" ];then
        net ads gpo pull -P $username $domainname $net_deb_path $pull_pkgs $local_pkg_path
    elif [ -z "$sharedpath" ] && [ -n "$pull_pkgs" ];then
        installed_pkgs=${uninstalled_pkgs//:/ }
	for pkg in $installed_pkgs
	do
	   is_exist=`echo $pull_pkgs | grep "$pkg"`
	    if [ -z "$is_exist" ];then
		will_installed=$will_installed':'$pkg
	    fi
	done

	uninstalled_pkgs=$will_installed
	pull_pkgs=
	ret="sharedpath is empty, pull packages failure!"
    else
	echo "no package need to be pulled from remote server."
    fi

    echo "*************stag3 : packages pulled*****************"
    pulled_pkgs=${pull_pkgs//:/ }
    for pkg in $pulled_pkgs
    do
        echo $pkg
    done
    if [ -n "$ret" ];then
	echo $ret
    fi
    echo "*****************************************************"


    if [[ $after_exception -eq 1 ]];then
        dpkg --configure -a
    fi
    installed_pkgs=${uninstalled_pkgs//:/ }
    for pkg in $installed_pkgs
    do    
        dpkg -i --force-overwrite $local_deb_path/$pkg/*.deb
    done

    echo "*************stage4 : packages installed*************"
    echo "repo packages installed:"
    if [ -z "repo_installed_pkgs" ];then
        echo "None!"
    else
        repo_installed_pkgs=${repo_installed_pkgs//:/ }
        for pkg in $repo_installed_pkgs
        do
	    echo $pkg
        done
    fi

    echo "deb packages installed:"
    if [ -z "$uninstalled_pkgs" ];then
        echo "None!"
    else
        installed_pkgs=${uninstalled_pkgs//:/ }
        for pkg in $installed_pkgs
        do
	    echo $pkg
        done
    fi

    echo "packages replaced:"
    if [ -z "$replaced_pkgs" ];then
        echo "None!"
    else
        rep_pkgs=${replaced_pkgs//;/ }
        for pkg in $rep_pkgs
        do
	    echo $pkg
        done
    fi
    echo "*****************************************************"
fi


#handle windows package installation
#if [[ -n $winapps ]];then
#    winpkgs=${winapps//:/ }
#    for pkg in $winpkgs
#    do
#        file_path=$local_win_path/$pkg
#	if [ ! -d $file_path ];then
#            mkdir -p $file_path
#	fi
#    done
#    net ads gpo pull -P $username $domainname $net_win_path $winapps $local_pkg_path
#    for pkg in $winpkgs
#    do
#	wine msiexec /i pkg
#	echo 'win install, winpkgs = '$winpkgs
#    done
#fi


#indicate that the process of instllation is sucessful.
mv $local_file_ver_bk $local_file_ver
echo "install packages successfully!"
