#!/bin/bash

conf_file=/etc/samba/policy.conf/software_uninstall_user.conf
local_pkg_path=/var/lib/samba/software_install
local_deb_path=$local_pkg_path/deb
local_win_path=$local_pkg_path/win
local_ver_path=$local_pkg_path/version
local_log_path=$local_pkg_path/log
local_file_ver=$local_ver_path/software_uninstall_user.version
local_file_ver_bk=$local_ver_path/software_uninstall_user.version.bk
local_log=$local_log_path/software_uninstall_user.log
flag_txt=txt

#get username
for param in "$@"
do
    tmp=${param%%=*}
    if [ "$tmp" = "--username" ];then
        username=${param#*=}
    fi
done

#get domainname
smbconf=`smbd -b | grep CONFIGFILE | cut -d : -f 2`
domainname=`grep -rnw 'realm' $smbconf | sed 's/ //g' | cut -d = -f 2`


if [ ! -d $local_ver_path ];then
    mkdir -p $local_ver_path
fi

if [ ! -d $local_log_path ];then
    mkdir -p $local_log_path
fi


#create log
cat /dev/null > $local_log

exec 1>$local_log
exec 2>$local_log

if [ ! -f $conf_file ];then
    echo "$conf_file not found, exit!"
    exit
fi

#if there no version number gived,exit
for param in `cat $conf_file`
do
    tmp=${param%%=*}
    if [ "$tmp" = "--VerNum" ];then
	net_ver=${param#*=}
    elif [ "$tmp" = "--SharedPath" ];then
	sharedpath=${param#*=}
    elif [ "$tmp" = "--RepoApps" ];then
        repoapps=${param#*=}
    elif [ "$tmp" = "--DebApps" ];then
        debapps=${param#*=}
    elif [ "$tmp" = "--WinApps" ];then
        winapps=${param#*=}
    fi
done

#verify whether net_ver meets the requirements or not
if [ -z "$net_ver" ] || [ -z "$(echo $net_ver | sed -n "/^[0-9]\+$/p")" ];then
    echo 'vernum does not meet the requirements,it sould be a number, exit'
    if [ -f $local_file_ver ];then
        rm -rf $local_file_ver
    fi
    exit
fi

#verify whether any package need to be installed or not
if [ -z "$repoapps" ] && [ -z "$debapps" ] && [ -z "$winapps" ];then
    echo "no package need to be uninstalled, exit."
    exit
fi


#determine if there is a policy update based on the version number
if [ -f $local_file_ver ];then
    local_ver=`cat $local_file_ver`
    echo 'local_ver = '$local_ver', net_ver = '$net_ver
    if [[ $net_ver = $local_ver ]];then
        echo '***stage1 : there is nothing changed, exit!**'
        exit
    elif [[ $net_ver != $local_ver ]];then
        echo '*****stage1 : something have changed!*******'
        echo $net_ver > $local_file_ver

	#for handling exception
	mv $local_file_ver $local_file_ver_bk
    fi
else
    echo '****stage1: there no version file, create it!***'
    echo $net_ver > $local_file_ver

    #for handling exception
    mv $local_file_ver $local_file_ver_bk
fi


##handle repoapps
if [ -n $repoapps ];then
    repoapps=${repoapps//:/ }
    for repo_pkg in $repoapps
    do
        apt-get -y purge $repo_pkg
	repo_purge_pkgs=$repo_purge_pkgs':'$repo_pkg
    done
fi


net_deb_path=$sharedpath\\deb
net_win_path=$sharedpath\\win

if [[ -n $debapps ]];then
    debpkgs=${debapps//:/ }
    for pkg in $debpkgs
    do
        file_path=$local_deb_path/$pkg
	if [ ! -d $file_path ];then
	    mkdir -p $file_path
	fi
    done

    if [ -n "$sharedpath" ];then

        #pull all lists.txt files needed from remote server
        echo 'net_deb_path = '$net_deb_path', debapps = '$debapps', local_pkg_path = '$local_pkg_path', flag_txt = '$flag_txt
	result=$(net ads gpo pull -P $username $domainname $net_deb_path $debapps $local_pkg_path $flag_txt)
	
	res=`echo $result | grep -w "gpo_fetch_packages failed"`
	if [ -n "$res" ];then
            echo "pull lists files failure, please check that sharedpath is correct or not!"
            mv $local_file_ver_bk $local_file_ver
	    exit
	fi
        echo '*****stage2: pull all lists.txt files sucessfully*****'
    else
	echo "sharedpath is null, we can't pull txt files, exit!"
	mv $local_file_ver_bk $local_file_ver
	exit
    fi

    for pkg in $debpkgs
    do
        txt_file=$local_deb_path/$pkg/lists.txt

	if [ -f "$txt_file" ];then
	    deb=`cat $txt_file`

	    #need_pull=0
	    for line in `cat $txt_file`
            do
		debname=${line%%_*}
		installed_name=`dpkg -l | awk '{print $2}' | grep -w "$debname"`
		if [ -z "$installed_name" ];then
		    echo "$debname has not been installed!"
		else
		    vernum=`dpkg -l | awk '{print $2, $3}' | grep -w "$debname" | grep -v "$debname-" | awk '{print $2}'`
		    txt_vernum=`dpkg-deb -f $local_deb_path/$pkg/$line | grep -w "Version:" | awk '{print $2}'`
		    echo "txt_vernum = $txt_vernum, vernum = $vernum"
		    if [ "$txt_vernum" = "$vernum" ];then
			apt-get -y purge $debname
		        echo "delete package: $debname"
		    fi
		    deb_uninstalled_pkgs=$deb_uninstalled_pkgs':'$debname
		fi
	    done
	fi
    done


    echo '*************stage4 : packages uninstalled*************'
    echo 'repo packages uninstalled:'
    repo_purge_pkgs=${repo_purge_pkgs//:/ }
    for pkg in $repo_purge_pkgs
    do
	echo $pkg
    done
    echo 'deb packages uninstalled:'
    deb_uninstalled_pkgs=${deb_uninstalled_pkgs//:/ }
    for pkg in $deb_uninstalled_pkgs
    do
	echo $pkg
    done
    echo '*****************************************************'
fi

##handle winapps uninstallation
#if [[ -n $winpkgs ]];then
#    for pkg in $winpkgs
#    do
#	echo 'win uninstall, winpkgs = '$winpkgs
#    done
#fi


#indicate that the process of instllation is sucessful.
mv $local_file_ver_bk $local_file_ver
echo 'uninstall packages successfully!'
